@@ -5,6 +5,8 @@ use utf8;
 
 our @EXPORT = qw/bulk_insert/;
 
+warn "IMPORTANT: Teng::Plugin::BulkInsert is DEPRECATED AND *WILL* BE REMOVED. DO NOT USE.\n";
+
 sub bulk_insert {
     my ($self, $table_name, $args) = @_;
 
@@ -25,7 +27,7 @@ sub bulk_insert {
         }
 
         my ($sql, @binds) = $self->sql_builder->insert_multi( $table_name, $args );
-        $self->_execute($sql, \@binds);
+        $self->execute($sql, \@binds);
     } else {
         # use transaction for better performance and atomicity.
         my $txn = $self->txn_scope();
@@ -42,7 +44,7 @@ __END__
 
 =head1 NAME
 
-Teng::Plugin::BulkInsert - bulk insert helper
+Teng::Plugin::BulkInsert - (DEPRECATED) bulk insert helper
 
 =head1 PROVIDED METHODS
 
