@@ -6,19 +6,12 @@ use utf8;
 our @EXPORT = qw/count/;
 
 sub count {
-    my ($self, $table, $column, $where) = @_;
+    my ($self, $table, $column, $where, $opt) = @_;
     $column ||= '*';
 
-    my $select = $self->sql_builder->new_select();
+    my ($sql, @binds) = $self->sql_builder->select($table, [\"COUNT($column)"], $where, $opt);
 
-    $select->add_select(\"COUNT($column)");
-    $select->add_from($table);
-    $select->add_where($_ => $where->{$_}) for keys %{ $where || {} };
-
-    my $sql = $select->as_sql();
-    my @bind = $select->bind();
-
-    my ($cnt) = $self->dbh->selectrow_array($sql, {}, @bind);
+    my ($cnt) = $self->dbh->selectrow_array($sql, {}, @binds);
     return $cnt;
 }
 
